@@ -22,7 +22,6 @@
 #include "IpHelper.h"
 #include "Jhash.h"
 #include "Oid.h"
-#include "Stt.h"
 #include "Switch.h"
 #include "User.h"
 #include "Vport.h"
@@ -628,6 +627,7 @@ HvDisconnectNic(POVS_SWITCH_CONTEXT switchContext,
     event.upcallPid = vport->upcallPid;
     RtlCopyMemory(&event.ovsName, &vport->ovsName, sizeof event.ovsName);
     event.type = OVS_EVENT_LINK_DOWN;
+    OvsPostVportEvent(&event);
 
     /*
      * Delete the port from the hash tables accessible to userspace. After this
@@ -635,13 +635,18 @@ HvDisconnectNic(POVS_SWITCH_CONTEXT switchContext,
      */
     if (OvsIsRealExternalVport(vport)) {
         OvsRemoveAndDeleteVport(NULL, switchContext, vport, FALSE, TRUE);
-        OvsPostVportEvent(&event);
     }
 
     if (isInternalPort) {
         OvsUnBindVportWithIpHelper(vport, switchContext);
-        OvsRemoveAndDeleteVport(NULL, switchContext, vport, TRUE, TRUE);
-        OvsPostVportEvent(&event);
+        /*
+         * Don't delete the port from the hash tables here for internal port
+         * because the internal port cannot be recreated in HvCreateNic(). It
+         * only can be created in HvCreatePort() by issuing
+         * OID_SWITCH_PORT_CREATE. We should wait extensible switch interface
+         * to issue OID_SWITCH_PORT_TEARDOWN and OID_SWITCH_PORT_DELETE to
+         * delete the internal port.
+         */
     }
     NdisReleaseRWLock(switchContext->dispatchLock, &lockState);
 
@@ -758,11 +763,6 @@ OvsFindTunnelVportByDstPortAndNWProto(POVS_SWITCH_CONTEXT switchContext,
                     continue;
                 }
                 break;
-            case IPPROTO_TCP:
-                if (vport->ovsType != OVS_VPORT_TYPE_STT) {
-                    continue;
-                }
-                break;
             case IPPROTO_GRE:
                 break;
             default:
@@ -1108,9 +1108,6 @@ OvsInitTunnelVport(PVOID userContext,
         }
         break;
     }
-    case OVS_VPORT_TYPE_STT:
-        status = OvsInitSttTunnel(vport, dstPort);
-        break;
     case OVS_VPORT_TYPE_GENEVE:
         status = OvsInitGeneveTunnel(vport, dstPort);
         break;
@@ -1248,7 +1245,6 @@ InitOvsVportCommon(POVS_SWITCH_CONTEXT switchContext,
     switch(vport->ovsType) {
     case OVS_VPORT_TYPE_GRE:
     case OVS_VPORT_TYPE_VXLAN:
-    case OVS_VPORT_TYPE_STT:
     case OVS_VPORT_TYPE_GENEVE:
     {
         UINT16 dstPort = GetPortFromPriv(vport);
@@ -1330,9 +1326,6 @@ OvsRemoveAndDeleteVport(PVOID usrParamsContext,
     case OVS_VPORT_TYPE_GENEVE:
         OvsCleanupGeneveTunnel(vport);
         break;
-    case OVS_VPORT_TYPE_STT:
-        OvsCleanupSttTunnel(vport);
-        break;
     case OVS_VPORT_TYPE_GRE:
         OvsCleanupGreTunnel(vport);
         break;
@@ -2293,10 +2286,6 @@ OvsNewVportCmdHandler(POVS_USER_PARAMS_CONTEXT usrParamsCtx,
             case OVS_VPORT_TYPE_GENEVE:
                 transportPortDest = GENEVE_UDP_PORT;
                 break;
-            case OVS_VPORT_TYPE_STT:
-                transportPortDest = STT_TCP_PORT;
-                nwProto = IPPROTO_TCP;
-                break;
             default:
                 nlError = NL_ERROR_INVAL;
                 goto Cleanup;
@@ -2415,9 +2404,6 @@ Cleanup:
                     case OVS_VPORT_TYPE_VXLAN:
                         OvsCleanupVxlanTunnel(NULL, vport, NULL, NULL);
                         break;
-                    case OVS_VPORT_TYPE_STT:
-                        OvsCleanupSttTunnel(vport);
-                        break;
                     case OVS_VPORT_TYPE_GENEVE:
                         OvsCleanupGeneveTunnel(vport);
                         break;
