@@ -202,6 +202,11 @@ enum ovs_packet_cmd {
  * @OVS_PACKET_ATTR_LEN: Packet size before truncation.
  * %OVS_PACKET_ATTR_USERSPACE action specify the Maximum received fragment
  * size.
+ * @OVS_PACKET_ATTR_UPCALL_PID: Netlink PID to use for upcalls while
+ * processing %OVS_PACKET_CMD_EXECUTE.  Takes precedence over all other ways
+ * to determine the Netlink PID including %OVS_USERSPACE_ATTR_PID,
+ * %OVS_DP_ATTR_UPCALL_PID, %OVS_DP_ATTR_PER_CPU_PIDS and the
+ * %OVS_VPORT_ATTR_UPCALL_PID.
  *
  * These attributes follow the &struct ovs_header within the Generic Netlink
  * payload for %OVS_PACKET_* commands.
@@ -221,6 +226,7 @@ enum ovs_packet_attr {
 	OVS_PACKET_ATTR_MRU,	    /* Maximum received IP fragment size. */
 	OVS_PACKET_ATTR_LEN,		/* Packet size before truncation. */
 	OVS_PACKET_ATTR_HASH,		/* Packet hash. */
+	OVS_PACKET_ATTR_UPCALL_PID, /* u32 Netlink PID. */
 	__OVS_PACKET_ATTR_MAX
 };
 
@@ -247,8 +253,8 @@ enum ovs_vport_type {
 	OVS_VPORT_TYPE_GRE,      /* GRE tunnel. */
 	OVS_VPORT_TYPE_VXLAN,	 /* VXLAN tunnel. */
 	OVS_VPORT_TYPE_GENEVE,	 /* Geneve tunnel. */
-	OVS_VPORT_TYPE_LISP = 105,  /* LISP tunnel (deprecated). */
-	OVS_VPORT_TYPE_STT = 106, /* STT tunnel (deprecated). */
+	/* OVS_VPORT_TYPE_LISP = 105,	LISP tunnel (no longer supported). */
+	/* OVS_VPORT_TYPE_STT = 106,	STT tunnel (no longer supported). */
 	OVS_VPORT_TYPE_ERSPAN = 107, /* ERSPAN tunnel. */
 	OVS_VPORT_TYPE_IP6ERSPAN = 108, /* ERSPAN tunnel. */
 	OVS_VPORT_TYPE_IP6GRE = 109,
@@ -465,6 +471,10 @@ enum xlate_error {
 	XLATE_UNSUPPORTED_PACKET_TYPE,
 	XLATE_CONGESTION_DROP,
 	XLATE_FORWARDING_DISABLED,
+	XLATE_TUNNEL_ROUTING_FAILED,
+	XLATE_TUNNEL_OUTPUT_NO_ETHERNET,
+	XLATE_TUNNEL_NEIGH_CACHE_MISS,
+	XLATE_TUNNEL_HEADER_BUILD_FAILED,
 	XLATE_MAX,
 };
 #endif
