@@ -51,16 +51,12 @@ dp_packet_gso_seg_new(const struct dp_packet *p, size_t hdr_len,
     seg->inner_l4_ofs = p->inner_l4_ofs;
 
     /* The protocol headers remain the same, so preserve hash and mark. */
+    seg->has_hash = p->has_hash;
     *dp_packet_rss_ptr(seg) = *dp_packet_rss_ptr(p);
+    seg->has_mark = p->has_mark;
     *dp_packet_flow_mark_ptr(seg) = *dp_packet_flow_mark_ptr(p);
 
-    /* The segment should inherit all the offloading flags from the
-     * original packet, except for the TCP segmentation, external
-     * buffer and indirect buffer flags. */
-    *dp_packet_ol_flags_ptr(seg) = *dp_packet_ol_flags_ptr(p)
-        & DP_PACKET_OL_SUPPORTED_MASK;
-
-    dp_packet_hwol_reset_tcp_seg(seg);
+    seg->offloads = p->offloads;
 
     return seg;
 }
@@ -73,7 +69,7 @@ dp_packet_gso_nr_segs(struct dp_packet *p)
     const char *data_tail;
     const char *data_pos;
 
-    if (dp_packet_hwol_is_tunnel(p)) {
+    if (dp_packet_tunnel(p)) {
         data_pos = dp_packet_get_inner_tcp_payload(p);
     } else {
         data_pos = dp_packet_get_tcp_payload(p);
@@ -95,6 +91,7 @@ dp_packet_gso(struct dp_packet *p, struct dp_packet_batch **batches)
     static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(1, 5);
     struct dp_packet_batch *curr_batch = *batches;
     struct tcp_header *tcp_hdr;
+    struct ip_header *ip_hdr;
     uint16_t inner_ip_id = 0;
     uint16_t outer_ip_id = 0;
     struct dp_packet *seg;
@@ -104,9 +101,9 @@ dp_packet_gso(struct dp_packet *p, struct dp_packet_batch **batches)
     bool outer_ipv4;
     int hdr_len;
     int seg_len;
-    bool udp_tnl = dp_packet_hwol_is_tunnel_vxlan(p) ||
-                   dp_packet_hwol_is_tunnel_geneve(p);
-    bool gre_tnl = dp_packet_hwol_is_tunnel_gre(p);
+    bool udp_tnl = dp_packet_tunnel_vxlan(p)
+                   || dp_packet_tunnel_geneve(p);
+    bool gre_tnl = dp_packet_tunnel_gre(p);
 
     tso_segsz = dp_packet_get_tso_segsz(p);
     if (!tso_segsz) {
@@ -116,24 +113,20 @@ dp_packet_gso(struct dp_packet *p, struct dp_packet_batch **batches)
     }
 
     if (udp_tnl || gre_tnl) {
-        outer_ipv4 = dp_packet_hwol_is_outer_ipv4(p);
-        tcp_hdr = dp_packet_inner_l4(p);
-
-        if (outer_ipv4) {
-            outer_ip_id = ntohs(((struct ip_header *) dp_packet_l3(p))->ip_id);
-        }
-        if (dp_packet_hwol_is_ipv4(p)) {
-            struct ip_header *ip_hdr = dp_packet_inner_l3(p);
+        ip_hdr = dp_packet_inner_l3(p);
+        if (IP_VER(ip_hdr->ip_ihl_ver) == 4) {
             inner_ip_id = ntohs(ip_hdr->ip_id);
         }
-    } else {
-        outer_ipv4 = dp_packet_hwol_is_ipv4(p);
-        tcp_hdr = dp_packet_l4(p);
 
-        if (outer_ipv4) {
-            struct ip_header *ip_hdr = dp_packet_l3(p);
-            outer_ip_id = ntohs(ip_hdr->ip_id);
-        }
+        tcp_hdr = dp_packet_inner_l4(p);
+    } else {
+        tcp_hdr = dp_packet_l4(p);
+    }
+
+    ip_hdr = dp_packet_l3(p);
+    outer_ipv4 = IP_VER(ip_hdr->ip_ihl_ver) == 4;
+    if (outer_ipv4) {
+        outer_ip_id = ntohs(ip_hdr->ip_id);
     }
 
     tcp_offset = TCP_OFFSET(tcp_hdr->tcp_ctl);
@@ -160,15 +153,17 @@ dp_packet_gso(struct dp_packet *p, struct dp_packet_batch **batches)
 
             tnl_hdr = dp_packet_l4(seg);
             tnl_hdr->udp_len = htons(dp_packet_l4_size(seg));
+            dp_packet_l4_checksum_set_partial(seg);
         }
 
         if (udp_tnl || gre_tnl) {
             /* Update tunnel inner L3 header. */
-            if (dp_packet_hwol_is_ipv4(seg)) {
-                struct ip_header *ip_hdr = dp_packet_inner_l3(seg);
+            ip_hdr = dp_packet_inner_l3(seg);
+            if (IP_VER(ip_hdr->ip_ihl_ver) == 4) {
                 ip_hdr->ip_tot_len = htons(dp_packet_inner_l3_size(seg));
                 ip_hdr->ip_id = htons(inner_ip_id);
                 ip_hdr->ip_csum = 0;
+                dp_packet_inner_ip_checksum_set_partial(seg);
                 inner_ip_id++;
             } else {
                 struct ovs_16aligned_ip6_hdr *ip6_hdr;
@@ -181,10 +176,11 @@ dp_packet_gso(struct dp_packet *p, struct dp_packet_batch **batches)
 
         /* Update L3 header. */
         if (outer_ipv4) {
-            struct ip_header *ip_hdr = dp_packet_l3(seg);
+            ip_hdr = dp_packet_l3(seg);
             ip_hdr->ip_tot_len = htons(dp_packet_l3_size(seg));
             ip_hdr->ip_id = htons(outer_ip_id);
             ip_hdr->ip_csum = 0;
+            dp_packet_ip_checksum_set_partial(seg);
             outer_ip_id++;
         } else {
             struct ovs_16aligned_ip6_hdr *ip6_hdr = dp_packet_l3(seg);
@@ -196,8 +192,10 @@ dp_packet_gso(struct dp_packet *p, struct dp_packet_batch **batches)
         /* Update L4 header. */
         if (udp_tnl || gre_tnl) {
             tcp_hdr = dp_packet_inner_l4(seg);
+            dp_packet_inner_l4_checksum_set_partial(seg);
         } else {
             tcp_hdr = dp_packet_l4(seg);
+            dp_packet_l4_checksum_set_partial(seg);
         }
         put_16aligned_be32(&tcp_hdr->tcp_seq, htonl(tcp_seq));
         tcp_seq += seg_len;
