@@ -39,9 +39,6 @@ dp_packet_init__(struct dp_packet *b, size_t allocated, enum dp_packet_source so
     dp_packet_init_specific(b);
     /* By default assume the packet type to be Ethernet. */
     b->packet_type = htonl(PT_ETH);
-    /* Reset csum start and offset. */
-    b->csum_start = 0;
-    b->csum_offset = 0;
 }
 
 static void
@@ -196,14 +193,11 @@ dp_packet_clone_with_headroom(const struct dp_packet *buffer, size_t headroom)
                                                     dp_packet_size(buffer),
                                                     headroom);
     /* Copy the following fields into the returned buffer: l2_pad_size,
-     * l2_5_ofs, l3_ofs, l4_ofs, cutlen, packet_type and md. */
+     * l2_5_ofs, l3_ofs, l4_ofs, cutlen, packet_type, offloads and md. */
     memcpy(&new_buffer->l2_pad_size, &buffer->l2_pad_size,
             sizeof(struct dp_packet) -
             offsetof(struct dp_packet, l2_pad_size));
 
-    *dp_packet_ol_flags_ptr(new_buffer) = *dp_packet_ol_flags_ptr(buffer);
-    *dp_packet_ol_flags_ptr(new_buffer) &= DP_PACKET_OL_SUPPORTED_MASK;
-
     dp_packet_set_tso_segsz(new_buffer, dp_packet_get_tso_segsz(buffer));
 
     if (dp_packet_rss_valid(buffer)) {
@@ -554,100 +548,92 @@ dp_packet_compare_offsets(struct dp_packet *b1, struct dp_packet *b2,
     return true;
 }
 
-void
-dp_packet_tnl_outer_ol_send_prepare(struct dp_packet *p,
-                                    uint64_t flags)
-{
-    if (dp_packet_hwol_is_outer_ipv4_cksum(p)) {
-        if (!(flags & NETDEV_TX_OFFLOAD_OUTER_IP_CKSUM)) {
-            dp_packet_ip_set_header_csum(p, false);
-            dp_packet_ol_set_ip_csum_good(p);
-            dp_packet_hwol_reset_outer_ipv4_csum(p);
-        }
-    }
-
-    if (!dp_packet_hwol_is_outer_udp_cksum(p)) {
-        return;
-    }
-
-    if (!(flags & NETDEV_TX_OFFLOAD_OUTER_UDP_CKSUM)) {
-        packet_udp_complete_csum(p, false);
-        dp_packet_ol_set_l4_csum_good(p);
-        dp_packet_hwol_reset_outer_udp_csum(p);
-    }
-}
-
 /* Checks if the packet 'p' is compatible with netdev_ol_flags 'flags'
  * and if not, updates the packet with the software fall back. */
 void
 dp_packet_ol_send_prepare(struct dp_packet *p, uint64_t flags)
 {
-    bool tnl_inner = false;
-
-    if (!dp_packet_hwol_tx_is_any_csum(p)) {
+    if (!dp_packet_ip_checksum_partial(p)
+        && !dp_packet_l4_checksum_partial(p)
+        && !dp_packet_inner_ip_checksum_partial(p)
+        && !dp_packet_inner_l4_checksum_partial(p)) {
         /* Only checksumming needs actions. */
         return;
     }
 
-    if (dp_packet_hwol_is_tunnel_geneve(p) ||
-        dp_packet_hwol_is_tunnel_vxlan(p)) {
-        tnl_inner = true;
+    if (!dp_packet_tunnel(p)) {
+        if (dp_packet_ip_checksum_partial(p)
+            && !(flags & NETDEV_TX_OFFLOAD_IPV4_CKSUM)) {
+            dp_packet_ip_set_header_csum(p, false);
+        }
+
+        if (dp_packet_l4_checksum_partial(p)) {
+            if (dp_packet_l4_proto_tcp(p)) {
+                if (!(flags & NETDEV_TX_OFFLOAD_TCP_CKSUM)) {
+                    packet_tcp_complete_csum(p, false);
+                }
+            } else if (dp_packet_l4_proto_udp(p)) {
+                if (!(flags & NETDEV_TX_OFFLOAD_UDP_CKSUM)) {
+                    packet_udp_complete_csum(p, false);
+                }
+            } else {
+                ovs_assert(dp_packet_l4_proto_sctp(p));
+                if (!(flags & NETDEV_TX_OFFLOAD_SCTP_CKSUM)) {
+                    packet_sctp_complete_csum(p, false);
+                }
+            }
+        }
+
+        return;
+    }
+
+    if (dp_packet_tunnel_geneve(p)
+        || dp_packet_tunnel_vxlan(p)) {
 
         /* If the TX interface doesn't support UDP tunnel offload but does
          * support inner checksum offload and an outer UDP checksum is
          * required, then we can't offload inner checksum either. As that would
          * invalidate the outer checksum. */
-        if (!(flags & NETDEV_TX_OFFLOAD_OUTER_UDP_CKSUM) &&
-                dp_packet_hwol_is_outer_udp_cksum(p)) {
+        if (!(flags & NETDEV_TX_OFFLOAD_OUTER_UDP_CKSUM)
+            && dp_packet_l4_checksum_partial(p)) {
             flags &= ~(NETDEV_TX_OFFLOAD_TCP_CKSUM |
                        NETDEV_TX_OFFLOAD_UDP_CKSUM |
                        NETDEV_TX_OFFLOAD_SCTP_CKSUM |
                        NETDEV_TX_OFFLOAD_IPV4_CKSUM);
         }
-    } else if (dp_packet_hwol_is_tunnel_gre(p)) {
-        tnl_inner = true;
     }
 
-    if (dp_packet_hwol_tx_ip_csum(p)) {
-        if (dp_packet_ip_checksum_good(p)) {
-            dp_packet_hwol_reset_tx_ip_csum(p);
-        } else if (!(flags & NETDEV_TX_OFFLOAD_IPV4_CKSUM)) {
-            dp_packet_ip_set_header_csum(p, tnl_inner);
-            dp_packet_ol_set_ip_csum_good(p);
-            dp_packet_hwol_reset_tx_ip_csum(p);
+    if (dp_packet_inner_ip_checksum_partial(p)
+        && !(flags & NETDEV_TX_OFFLOAD_IPV4_CKSUM)) {
+        dp_packet_ip_set_header_csum(p, true);
+    }
+
+    if (dp_packet_inner_l4_checksum_partial(p)) {
+        if (dp_packet_inner_l4_proto_tcp(p)) {
+            if (!(flags & NETDEV_TX_OFFLOAD_TCP_CKSUM)) {
+                packet_tcp_complete_csum(p, true);
+            }
+        } else if (dp_packet_inner_l4_proto_udp(p)) {
+            if (!(flags & NETDEV_TX_OFFLOAD_UDP_CKSUM)) {
+                packet_udp_complete_csum(p, true);
+            }
+        } else {
+            ovs_assert(dp_packet_inner_l4_proto_sctp(p));
+            if (!(flags & NETDEV_TX_OFFLOAD_SCTP_CKSUM)) {
+                packet_sctp_complete_csum(p, true);
+            }
         }
     }
 
-    if (!dp_packet_hwol_tx_l4_checksum(p)) {
-        if (tnl_inner) {
-            dp_packet_tnl_outer_ol_send_prepare(p, flags);
+    if (dp_packet_ip_checksum_partial(p)
+        && !(flags & NETDEV_TX_OFFLOAD_OUTER_IP_CKSUM)) {
+        dp_packet_ip_set_header_csum(p, false);
+    }
+
+    if (dp_packet_l4_checksum_partial(p)) {
+        ovs_assert(dp_packet_l4_proto_udp(p));
+        if (!(flags & NETDEV_TX_OFFLOAD_OUTER_UDP_CKSUM)) {
+            packet_udp_complete_csum(p, false);
         }
-        return;
-    }
-
-    if (dp_packet_l4_checksum_good(p) && !tnl_inner) {
-        dp_packet_hwol_reset_tx_l4_csum(p);
-        return;
-    }
-
-    if (dp_packet_hwol_l4_is_tcp(p)
-        && !(flags & NETDEV_TX_OFFLOAD_TCP_CKSUM)) {
-        packet_tcp_complete_csum(p, tnl_inner);
-        dp_packet_ol_set_l4_csum_good(p);
-        dp_packet_hwol_reset_tx_l4_csum(p);
-    } else if (dp_packet_hwol_l4_is_udp(p)
-               && !(flags & NETDEV_TX_OFFLOAD_UDP_CKSUM)) {
-        packet_udp_complete_csum(p, tnl_inner);
-        dp_packet_ol_set_l4_csum_good(p);
-        dp_packet_hwol_reset_tx_l4_csum(p);
-    } else if (!(flags & NETDEV_TX_OFFLOAD_SCTP_CKSUM)
-               && dp_packet_hwol_l4_is_sctp(p)) {
-        packet_sctp_complete_csum(p, tnl_inner);
-        dp_packet_ol_set_l4_csum_good(p);
-        dp_packet_hwol_reset_tx_l4_csum(p);
-    }
-
-    if (tnl_inner) {
-        dp_packet_tnl_outer_ol_send_prepare(p, flags);
     }
 }
