@@ -758,41 +758,6 @@ mfex_check_tcp_data_offset(const struct tcp_header *tcp)
     return ret;
 }
 
-static void
-mfex_ipv4_set_hwol(struct dp_packet *pkt)
-{
-    dp_packet_hwol_set_tx_ipv4(pkt);
-    if (dp_packet_ip_checksum_good(pkt)) {
-        dp_packet_hwol_set_tx_ip_csum(pkt);
-    }
-}
-
-static void
-mfex_ipv6_set_hwol(struct dp_packet *pkt)
-{
-    dp_packet_hwol_set_tx_ipv6(pkt);
-}
-
-static void
-mfex_tcp_set_hwol(struct dp_packet *pkt)
-{
-    dp_packet_ol_l4_csum_check_partial(pkt);
-    if (dp_packet_l4_checksum_good(pkt)
-        || dp_packet_ol_l4_csum_partial(pkt)) {
-        dp_packet_hwol_set_csum_tcp(pkt);
-    }
-}
-
-static void
-mfex_udp_set_hwol(struct dp_packet *pkt)
-{
-    dp_packet_ol_l4_csum_check_partial(pkt);
-    if (dp_packet_l4_checksum_good(pkt)
-        || dp_packet_ol_l4_csum_partial(pkt)) {
-        dp_packet_hwol_set_csum_udp(pkt);
-    }
-}
-
 /* Generic loop to process any mfex profile. This code is specialized into
  * multiple actual MFEX implementation functions. Its marked ALWAYS_INLINE
  * to ensure the compiler specializes each instance. The code is marked "hot"
@@ -894,8 +859,7 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                 const struct tcp_header *tcp = (void *)&pkt[38];
                 mfex_handle_tcp_flags(tcp, &blocks[7]);
                 dp_packet_update_rss_hash_ipv4_tcp_udp(packet);
-                mfex_ipv4_set_hwol(packet);
-                mfex_tcp_set_hwol(packet);
+                dp_packet_l4_proto_set_tcp(packet);
             } break;
 
         case PROFILE_ETH_VLAN_IPV4_UDP: {
@@ -908,8 +872,7 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                     continue;
                 }
                 dp_packet_update_rss_hash_ipv4_tcp_udp(packet);
-                mfex_ipv4_set_hwol(packet);
-                mfex_udp_set_hwol(packet);
+                dp_packet_l4_proto_set_udp(packet);
             } break;
 
         case PROFILE_ETH_IPV4_TCP: {
@@ -925,8 +888,7 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                     continue;
                 }
                 dp_packet_update_rss_hash_ipv4_tcp_udp(packet);
-                mfex_ipv4_set_hwol(packet);
-                mfex_tcp_set_hwol(packet);
+                dp_packet_l4_proto_set_tcp(packet);
             } break;
 
         case PROFILE_ETH_IPV4_UDP: {
@@ -938,8 +900,7 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                     continue;
                 }
                 dp_packet_update_rss_hash_ipv4_tcp_udp(packet);
-                mfex_ipv4_set_hwol(packet);
-                mfex_udp_set_hwol(packet);
+                dp_packet_l4_proto_set_udp(packet);
             } break;
 
         case PROFILE_ETH_IPV6_UDP: {
@@ -958,8 +919,7 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                 /* Process UDP header. */
                 mfex_handle_ipv6_l4((void *)&pkt[54], &blocks[9]);
                 dp_packet_update_rss_hash_ipv6_tcp_udp(packet);
-                mfex_ipv6_set_hwol(packet);
-                mfex_udp_set_hwol(packet);
+                dp_packet_l4_proto_set_udp(packet);
             } break;
 
         case PROFILE_ETH_IPV6_TCP: {
@@ -983,8 +943,7 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                 }
                 mfex_handle_tcp_flags(tcp, &blocks[9]);
                 dp_packet_update_rss_hash_ipv6_tcp_udp(packet);
-                mfex_ipv6_set_hwol(packet);
-                mfex_tcp_set_hwol(packet);
+                dp_packet_l4_proto_set_tcp(packet);
             } break;
 
         case PROFILE_ETH_VLAN_IPV6_TCP: {
@@ -1011,8 +970,7 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                 }
                 mfex_handle_tcp_flags(tcp, &blocks[10]);
                 dp_packet_update_rss_hash_ipv6_tcp_udp(packet);
-                mfex_ipv6_set_hwol(packet);
-                mfex_tcp_set_hwol(packet);
+                dp_packet_l4_proto_set_tcp(packet);
             } break;
 
         case PROFILE_ETH_VLAN_IPV6_UDP: {
@@ -1034,8 +992,7 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                 /* Process UDP header. */
                 mfex_handle_ipv6_l4((void *)&pkt[58], &blocks[10]);
                 dp_packet_update_rss_hash_ipv6_tcp_udp(packet);
-                mfex_ipv6_set_hwol(packet);
-                mfex_udp_set_hwol(packet);
+                dp_packet_l4_proto_set_udp(packet);
             } break;
 
         case PROFILE_ETH_IPV4_NVGRE: {
@@ -1046,8 +1003,7 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                     continue;
                 }
                 dp_packet_update_rss_hash_ipv4(packet);
-                mfex_ipv4_set_hwol(packet);
-                mfex_udp_set_hwol(packet);
+                dp_packet_l4_proto_set_udp(packet);
             } break;
 
         default:
