@@ -135,8 +135,6 @@ vport_type_to_kind(enum ovs_vport_type type,
         return "bareudp";
     case OVS_VPORT_TYPE_NETDEV:
     case OVS_VPORT_TYPE_INTERNAL:
-    case OVS_VPORT_TYPE_LISP:
-    case OVS_VPORT_TYPE_STT:
     case OVS_VPORT_TYPE_UNSPEC:
     case __OVS_VPORT_TYPE_MAX:
     default:
@@ -318,8 +316,6 @@ dpif_netlink_rtnl_verify(const struct netdev_tunnel_config *tnl_cfg,
         break;
     case OVS_VPORT_TYPE_NETDEV:
     case OVS_VPORT_TYPE_INTERNAL:
-    case OVS_VPORT_TYPE_LISP:
-    case OVS_VPORT_TYPE_STT:
     case OVS_VPORT_TYPE_GTPU:
     case OVS_VPORT_TYPE_SRV6:
     case OVS_VPORT_TYPE_UNSPEC:
@@ -411,8 +407,6 @@ dpif_netlink_rtnl_create(const struct netdev_tunnel_config *tnl_cfg,
         break;
     case OVS_VPORT_TYPE_NETDEV:
     case OVS_VPORT_TYPE_INTERNAL:
-    case OVS_VPORT_TYPE_LISP:
-    case OVS_VPORT_TYPE_STT:
     case OVS_VPORT_TYPE_GTPU:
     case OVS_VPORT_TYPE_SRV6:
     case OVS_VPORT_TYPE_UNSPEC:
@@ -528,8 +522,6 @@ dpif_netlink_rtnl_port_destroy(const char *name, const char *type)
         return dpif_netlink_rtnl_destroy(name);
     case OVS_VPORT_TYPE_NETDEV:
     case OVS_VPORT_TYPE_INTERNAL:
-    case OVS_VPORT_TYPE_LISP:
-    case OVS_VPORT_TYPE_STT:
     case OVS_VPORT_TYPE_GTPU:
     case OVS_VPORT_TYPE_UNSPEC:
     case __OVS_VPORT_TYPE_MAX:
