@@ -732,6 +732,8 @@ struct dpif_execute {
     unsigned int mtu;               /* Maximum transmission unit to fragment.
                                        0 if not a fragmented packet */
     uint64_t hash;                  /* Packet flow hash. 0 if not specified. */
+    uint64_t upcall_pid;            /* Netlink PID to use for upcalls.
+                                     * 0 if not specified. */
     const struct flow *flow;         /* Flow extracted from 'packet'. */
 
     /* Input, but possibly modified as a side effect of execution. */
@@ -833,6 +835,7 @@ struct dpif_upcall {
     struct nlattr *mru;         /* Maximum receive unit. */
     struct nlattr *hash;        /* Packet hash. */
     struct nlattr *cutlen;      /* Number of bytes shrink from the end. */
+    uint32_t pid;               /* Socket PID the upcall was received from. */
 
     /* DPIF_UC_ACTION only. */
     struct nlattr *userdata;    /* Argument to OVS_ACTION_ATTR_USERSPACE. */
