@@ -437,8 +437,8 @@ ipf_reassemble_v4_frags(struct ipf_list *ipf_list)
     len += rest_len;
     l3 = dp_packet_l3(pkt);
     ovs_be16 new_ip_frag_off = l3->ip_frag_off & ~htons(IP_MORE_FRAGMENTS);
-    if (dp_packet_hwol_tx_ip_csum(pkt)) {
-        dp_packet_ol_reset_ip_csum_good(pkt);
+    if (dp_packet_ip_checksum_valid(pkt)) {
+        dp_packet_ip_checksum_set_partial(pkt);
     } else {
         l3->ip_csum = recalc_csum16(l3->ip_csum, l3->ip_frag_off,
                                     new_ip_frag_off);
@@ -616,9 +616,14 @@ ipf_is_valid_v4_frag(struct ipf *ipf, struct dp_packet *pkt)
     }
 
     bool bad_csum = dp_packet_ip_checksum_bad(pkt);
-    if (OVS_UNLIKELY(!bad_csum && !dp_packet_ip_checksum_good(pkt))) {
+    if (OVS_UNLIKELY(!bad_csum && dp_packet_ip_checksum_unknown(pkt))) {
         COVERAGE_INC(ipf_l3csum_checked);
-        bad_csum = csum(l3, ip_hdr_len);
+        if (csum(l3, ip_hdr_len)) {
+            dp_packet_ip_checksum_set_bad(pkt);
+            bad_csum = true;
+        } else {
+            dp_packet_ip_checksum_set_good(pkt);
+        }
     }
     if (OVS_UNLIKELY(bad_csum)) {
         COVERAGE_INC(ipf_l3csum_err);
@@ -1188,6 +1193,9 @@ ipf_post_execute_reass_pkts(struct ipf *ipf,
                 const struct ipf_frag *frag_0 = &rp->list->frag_list[0];
                 void *l4_frag = dp_packet_l4(frag_0->pkt);
                 void *l4_reass = dp_packet_l4(pkt);
+
+                /* Complete all L4 checksums before reassembly. */
+                dp_packet_ol_send_prepare(pkt, 0);
                 memcpy(l4_frag, l4_reass, dp_packet_l4_size(frag_0->pkt));
 
                 for (int i = 0; i <= rp->list->last_inuse_idx; i++) {
@@ -1216,8 +1224,8 @@ ipf_post_execute_reass_pkts(struct ipf *ipf,
                     } else {
                         struct ip_header *l3_frag = dp_packet_l3(frag_i->pkt);
                         struct ip_header *l3_reass = dp_packet_l3(pkt);
-                        if (dp_packet_hwol_tx_ip_csum(frag_i->pkt)) {
-                            dp_packet_ol_reset_ip_csum_good(frag_i->pkt);
+                        if (dp_packet_ip_checksum_valid(frag_i->pkt)) {
+                            dp_packet_ip_checksum_set_partial(frag_i->pkt);
                         } else {
                             ovs_be32 reass_ip =
                                 get_16aligned_be32(&l3_reass->ip_src);
