@@ -118,10 +118,14 @@ ip_extract_tnl_md(struct dp_packet *packet, struct flow_tnl *tnl,
 
         /* A packet coming from a network device might have the
          * csum already checked. In this case, skip the check. */
-        if (OVS_UNLIKELY(!bad_csum
-                         && !dp_packet_hwol_l3_csum_ipv4_ol(packet))) {
+        if (OVS_UNLIKELY(!bad_csum && dp_packet_ip_checksum_unknown(packet))) {
             COVERAGE_INC(native_tnl_l3csum_checked);
-            bad_csum = csum(ip, IP_IHL(ip->ip_ihl_ver) * 4);
+            if (csum(ip, IP_IHL(ip->ip_ihl_ver) * 4)) {
+                dp_packet_ip_checksum_set_bad(packet);
+                bad_csum = true;
+            } else {
+                dp_packet_ip_checksum_set_good(packet);
+            }
         }
         if (OVS_UNLIKELY(bad_csum)) {
             COVERAGE_INC(native_tnl_l3csum_err);
@@ -202,31 +206,20 @@ netdev_tnl_push_ip_header(struct dp_packet *packet, const void *header,
         *ip_tot_size -= IPV6_HEADER_LEN;
         ip6->ip6_plen = htons(*ip_tot_size);
         packet_set_ipv6_flow_label(&ip6->ip6_flow, ipv6_label);
+        dp_packet_ip_checksum_set_unknown(packet);
+
         packet->l4_ofs = dp_packet_size(packet) - *ip_tot_size;
 
-        if (dp_packet_hwol_is_tunnel(packet)) {
-            dp_packet_hwol_set_tx_outer_ipv6(packet);
-        } else {
-            dp_packet_hwol_set_tx_ipv6(packet);
-        }
-
-        dp_packet_ol_reset_ip_csum_good(packet);
         return ip6 + 1;
     } else {
         ip = netdev_tnl_ip_hdr(eth);
         ip->ip_tot_len = htons(*ip_tot_size);
-        /* Postpone checksum to when the packet is pushed to the port. */
-        if (dp_packet_hwol_is_tunnel(packet)) {
-            dp_packet_hwol_set_tx_outer_ipv4(packet);
-            dp_packet_hwol_set_tx_outer_ipv4_csum(packet);
-        } else {
-            dp_packet_hwol_set_tx_ipv4(packet);
-            dp_packet_hwol_set_tx_ip_csum(packet);
-        }
-
-        dp_packet_ol_reset_ip_csum_good(packet);
         *ip_tot_size -= IP_HEADER_LEN;
+        /* Postpone checksum to when the packet is pushed to the port. */
+        dp_packet_ip_checksum_set_partial(packet);
+
         packet->l4_ofs = dp_packet_size(packet) - *ip_tot_size;
+
         return ip + 1;
     }
 }
@@ -245,8 +238,7 @@ udp_extract_tnl_md(struct dp_packet *packet, struct flow_tnl *tnl,
     if (udp->udp_csum) {
         bool bad_csum = dp_packet_l4_checksum_bad(packet);
 
-        if (OVS_LIKELY(!bad_csum && !dp_packet_ol_l4_csum_partial(packet))
-            && OVS_UNLIKELY(!dp_packet_l4_checksum_good(packet))) {
+        if (OVS_UNLIKELY(!bad_csum && dp_packet_l4_checksum_unknown(packet))) {
             uint32_t csum;
             COVERAGE_INC(native_tnl_l4csum_checked);
             if (netdev_tnl_is_header_ipv6(dp_packet_data(packet))) {
@@ -259,7 +251,12 @@ udp_extract_tnl_md(struct dp_packet *packet, struct flow_tnl *tnl,
                                  ((const unsigned char *)udp -
                                   (const unsigned char *)dp_packet_eth(packet)
                                  ));
-            bad_csum = csum_finish(csum);
+            if (csum_finish(csum)) {
+                dp_packet_l4_checksum_set_bad(packet);
+                bad_csum = true;
+            } else {
+                dp_packet_l4_checksum_set_good(packet);
+            }
         }
         if (OVS_UNLIKELY(bad_csum)) {
             COVERAGE_INC(native_tnl_l4csum_err);
@@ -275,38 +272,29 @@ udp_extract_tnl_md(struct dp_packet *packet, struct flow_tnl *tnl,
 }
 
 static void
-dp_packet_tnl_ol_process(struct dp_packet *packet,
-                         const struct ovs_action_push_tnl *data)
+tnl_ol_push(struct dp_packet *packet,
+            const struct ovs_action_push_tnl *data)
 {
-    struct ip_header *ip = NULL;
-
-    if (dp_packet_hwol_l4_mask(packet)) {
-        ip = dp_packet_l3(packet);
-
-        if (data->tnl_type == OVS_VPORT_TYPE_GENEVE ||
-            data->tnl_type == OVS_VPORT_TYPE_VXLAN ||
-            data->tnl_type == OVS_VPORT_TYPE_GRE ||
-            data->tnl_type == OVS_VPORT_TYPE_IP6GRE) {
-
-            if (IP_VER(ip->ip_ihl_ver) == 4) {
-                dp_packet_hwol_set_tx_ipv4(packet);
-                dp_packet_hwol_set_tx_ip_csum(packet);
-            } else if (IP_VER(ip->ip_ihl_ver) == 6) {
-                dp_packet_hwol_set_tx_ipv6(packet);
-            }
-        }
-    }
+    packet->offloads <<= DP_PACKET_OL_SHIFT_COUNT;
 
     if (data->tnl_type == OVS_VPORT_TYPE_GENEVE) {
-        dp_packet_hwol_set_tunnel_geneve(packet);
+        dp_packet_tunnel_set_geneve(packet);
     } else if (data->tnl_type == OVS_VPORT_TYPE_VXLAN) {
-        dp_packet_hwol_set_tunnel_vxlan(packet);
+        dp_packet_tunnel_set_vxlan(packet);
     } else if (data->tnl_type == OVS_VPORT_TYPE_GRE ||
                data->tnl_type == OVS_VPORT_TYPE_IP6GRE) {
-        dp_packet_hwol_set_tunnel_gre(packet);
+        dp_packet_tunnel_set_gre(packet);
     }
 }
 
+static void
+tnl_ol_pop(struct dp_packet *packet, int off)
+{
+    packet->offloads >>= DP_PACKET_OL_SHIFT_COUNT;
+
+    dp_packet_reset_packet(packet, off);
+}
+
 void
 netdev_tnl_push_udp_header(const struct netdev *netdev OVS_UNUSED,
                            struct dp_packet *packet,
@@ -322,27 +310,18 @@ netdev_tnl_push_udp_header(const struct netdev *netdev OVS_UNUSED,
      * modifying the packet. */
     udp_src = netdev_tnl_get_src_port(packet);
 
-    dp_packet_tnl_ol_process(packet, data);
+    tnl_ol_push(packet, data);
     udp = netdev_tnl_push_ip_header(packet, data->header, data->header_len,
                                     &ip_tot_size, 0);
 
     udp->udp_src = udp_src;
     udp->udp_len = htons(ip_tot_size);
 
+    dp_packet_l4_proto_set_udp(packet);
     if (udp->udp_csum) {
-        dp_packet_ol_reset_l4_csum_good(packet);
-        if (dp_packet_hwol_is_tunnel_geneve(packet) ||
-            dp_packet_hwol_is_tunnel_vxlan(packet)) {
-            dp_packet_hwol_set_outer_udp_csum(packet);
-        } else {
-            dp_packet_hwol_set_csum_udp(packet);
-        }
-    }
-
-    if (packet->csum_start && packet->csum_offset) {
-        dp_packet_ol_set_l4_csum_partial(packet);
-    } else if (!udp->udp_csum) {
-        dp_packet_ol_set_l4_csum_good(packet);
+        dp_packet_l4_checksum_set_partial(packet);
+    } else {
+        dp_packet_l4_checksum_set_good(packet);
     }
 
     if (l3_ofs != UINT16_MAX) {
@@ -539,7 +518,7 @@ netdev_gre_pop_header(struct dp_packet *packet)
         goto err;
     }
 
-    dp_packet_reset_packet(packet, hlen);
+    tnl_ol_pop(packet, hlen);
 
     return packet;
 err:
@@ -558,7 +537,7 @@ netdev_gre_push_header(const struct netdev *netdev,
     struct gre_base_hdr *greh;
     int ip_tot_size;
 
-    dp_packet_tnl_ol_process(packet, data);
+    tnl_ol_push(packet, data);
 
     greh = netdev_tnl_push_ip_header(packet, data->header, data->header_len,
                                      &ip_tot_size, 0);
@@ -569,7 +548,7 @@ netdev_gre_push_header(const struct netdev *netdev,
     }
 
     if (greh->flags & htons(GRE_SEQ)) {
-        if (!dp_packet_hwol_is_tso(packet)) {
+        if (!dp_packet_get_tso_segsz(packet)) {
             /* Last 4 bytes are GRE seqno. */
             int seq_ofs = gre_header_len(greh->flags) - 4;
             ovs_16aligned_be32 *seq_opt =
@@ -702,7 +681,7 @@ netdev_erspan_pop_header(struct dp_packet *packet)
         goto err;
     }
 
-    dp_packet_reset_packet(packet, hlen);
+    tnl_ol_pop(packet, hlen);
 
     return packet;
 err:
@@ -864,7 +843,7 @@ netdev_gtpu_pop_header(struct dp_packet *packet)
         } else {
             VLOG_WARN_RL(&err_rl, "GTP-U: Receive non-IP packet.");
         }
-        dp_packet_reset_packet(packet, hlen + gtpu_hlen);
+        tnl_ol_pop(packet, hlen + gtpu_hlen);
     } else {
         /* non-GPDU GTP-U messages, ex: echo request, end marker.
          * Users should redirect these packets to controller, or.
@@ -905,8 +884,8 @@ netdev_gtpu_push_header(const struct netdev *netdev,
     udp->udp_src = udp_src;
     udp->udp_len = htons(ip_tot_size);
     /* Postpone checksum to the egress netdev. */
-    dp_packet_hwol_set_csum_udp(packet);
-    dp_packet_ol_reset_l4_csum_good(packet);
+    dp_packet_l4_proto_set_udp(packet);
+    dp_packet_l4_checksum_set_partial(packet);
 
     gtpuh = ALIGNED_CAST(struct gtpuhdr *, udp + 1);
 
@@ -1102,7 +1081,7 @@ netdev_srv6_pop_header(struct dp_packet *packet)
         goto err;
     }
 
-    dp_packet_reset_packet(packet, hlen);
+    tnl_ol_pop(packet, hlen);
 
     return packet;
 err:
@@ -1169,7 +1148,7 @@ netdev_vxlan_pop_header(struct dp_packet *packet)
     tnl->flags |= FLOW_TNL_F_KEY;
 
     packet->packet_type = htonl(next_pt);
-    dp_packet_reset_packet(packet, hlen + VXLAN_HLEN);
+    tnl_ol_pop(packet, hlen + VXLAN_HLEN);
     if (next_pt != PT_ETH) {
         packet->l3_ofs = 0;
     }
@@ -1277,7 +1256,7 @@ netdev_geneve_pop_header(struct dp_packet *packet)
     tnl->flags |= FLOW_TNL_F_UDPIF;
 
     packet->packet_type = htonl(PT_ETH);
-    dp_packet_reset_packet(packet, hlen);
+    tnl_ol_pop(packet, hlen);
 
     return packet;
 err:
