@@ -56,6 +56,10 @@ COVERAGE_DEFINE(drop_action_invalid_tunnel_metadata);
 COVERAGE_DEFINE(drop_action_unsupported_packet_type);
 COVERAGE_DEFINE(drop_action_congestion);
 COVERAGE_DEFINE(drop_action_forwarding_disabled);
+COVERAGE_DEFINE(drop_action_tunnel_routing_failed);
+COVERAGE_DEFINE(drop_action_tunnel_output_no_ethernet);
+COVERAGE_DEFINE(drop_action_tunnel_neigh_cache_miss);
+COVERAGE_DEFINE(drop_action_tunnel_header_build_failed);
 
 static void
 dp_update_drop_action_counter(enum xlate_error drop_reason,
@@ -100,6 +104,18 @@ dp_update_drop_action_counter(enum xlate_error drop_reason,
    case XLATE_FORWARDING_DISABLED:
         COVERAGE_ADD(drop_action_forwarding_disabled, delta);
         break;
+   case XLATE_TUNNEL_ROUTING_FAILED:
+        COVERAGE_ADD(drop_action_tunnel_routing_failed, delta);
+        break;
+   case XLATE_TUNNEL_OUTPUT_NO_ETHERNET:
+        COVERAGE_ADD(drop_action_tunnel_output_no_ethernet, delta);
+        break;
+   case XLATE_TUNNEL_NEIGH_CACHE_MISS:
+        COVERAGE_ADD(drop_action_tunnel_neigh_cache_miss, delta);
+        break;
+   case XLATE_TUNNEL_HEADER_BUILD_FAILED:
+        COVERAGE_ADD(drop_action_tunnel_header_build_failed, delta);
+        break;
    case XLATE_MAX:
    default:
         VLOG_ERR_RL(&rl, "Invalid Drop reason type: %d", drop_reason);
@@ -171,8 +187,8 @@ odp_set_ipv4(struct dp_packet *packet, const struct ovs_key_ipv4 *key,
         new_tos = key->ipv4_tos | (nh->ip_tos & ~mask->ipv4_tos);
 
         if (nh->ip_tos != new_tos) {
-            if (dp_packet_hwol_tx_ip_csum(packet)) {
-                dp_packet_ol_reset_ip_csum_good(packet);
+            if (dp_packet_ip_checksum_valid(packet)) {
+                dp_packet_ip_checksum_set_partial(packet);
             } else {
                 nh->ip_csum = recalc_csum16(nh->ip_csum,
                                             htons((uint16_t) nh->ip_tos),
@@ -187,8 +203,8 @@ odp_set_ipv4(struct dp_packet *packet, const struct ovs_key_ipv4 *key,
         new_ttl = key->ipv4_ttl | (nh->ip_ttl & ~mask->ipv4_ttl);
 
         if (OVS_LIKELY(nh->ip_ttl != new_ttl)) {
-            if (dp_packet_hwol_tx_ip_csum(packet)) {
-                dp_packet_ol_reset_ip_csum_good(packet);
+            if (dp_packet_ip_checksum_valid(packet)) {
+                dp_packet_ip_checksum_set_partial(packet);
             } else {
                 nh->ip_csum = recalc_csum16(nh->ip_csum,
                                             htons(nh->ip_ttl << 8),
