@@ -301,3 +301,37 @@ xlate_cache_steal_entries(struct xlate_cache *dst, struct xlate_cache *src)
     memcpy(p, src_entries->data, src_entries->size);
     ofpbuf_clear(src_entries);
 }
+
+void
+xlate_xcache_format(struct ds *s, const struct xlate_cache *xcache)
+{
+    struct ofpbuf entries = xcache->entries;
+    struct xc_entry *entry;
+    struct ofgroup *ofg;
+
+    XC_ENTRY_FOR_EACH (entry, &entries) {
+        switch (entry->type) {
+        case XC_RULE:
+            ofproto_rule_stats_ds(s, &entry->rule->up, true);
+            break;
+        case XC_GROUP:
+            ofg = &entry->group.group->up;
+            ofputil_group_format(s, ofg->group_id, ofg->type,
+                                 entry->group.bucket, &ofg->buckets,
+                                 &ofg->props, OFP15_VERSION,
+                                 false, NULL, NULL);
+            break;
+        case XC_TABLE:
+        case XC_BOND:
+        case XC_NETDEV:
+        case XC_NETFLOW:
+        case XC_MIRROR:
+        case XC_LEARN:
+        case XC_NORMAL:
+        case XC_FIN_TIMEOUT:
+        case XC_TNL_NEIGH:
+        case XC_TUNNEL_HEADER:
+            break;
+        }
+    }
+}
