@@ -23,6 +23,7 @@
 
 #include "bitmap.h"
 #include "column.h"
+#include "cooperative-multitasking.h"
 #include "log.h"
 #include "openvswitch/json.h"
 #include "lockfile.h"
@@ -80,8 +81,8 @@ ovsdb_file_column_diff_disable(void)
 }
 
 static struct ovsdb_error *
-ovsdb_file_update_row_from_json(struct ovsdb_row *row, bool converting,
-                                bool row_contains_diff,
+ovsdb_file_update_row_from_json(struct ovsdb_row *row, struct ovsdb_row *diff,
+                                bool converting, bool row_contains_diff,
                                 const struct json *json)
 {
     struct ovsdb_table_schema *schema = row->table->schema;
@@ -122,6 +123,12 @@ ovsdb_file_update_row_from_json(struct ovsdb_row *row, bool converting,
             error = ovsdb_datum_apply_diff_in_place(
                                            &row->fields[column->index],
                                            &datum, &column->type);
+            if (!error && diff) {
+                ovs_assert(ovsdb_datum_is_default(&diff->fields[column->index],
+                                                  &column->type));
+                ovsdb_datum_swap(&diff->fields[column->index], &datum);
+            }
+
             ovsdb_datum_destroy(&datum, &column->type);
             if (error) {
                 return error;
@@ -150,16 +157,20 @@ ovsdb_file_txn_row_from_json(struct ovsdb_txn *txn, struct ovsdb_table *table,
         ovsdb_txn_row_delete(txn, row);
         return NULL;
     } else if (row) {
-        return ovsdb_file_update_row_from_json(ovsdb_txn_row_modify(txn, row),
-                                               converting, row_contains_diff,
-                                               json);
+        struct ovsdb_row *new, *diff = NULL;
+
+        ovsdb_txn_row_modify(txn, row, &new,
+                             row_contains_diff ? &diff : NULL);
+        return ovsdb_file_update_row_from_json(new, diff, converting,
+                                               row_contains_diff, json);
     } else {
         struct ovsdb_error *error;
         struct ovsdb_row *new;
 
         new = ovsdb_row_create(table);
         *ovsdb_row_get_uuid_rw(new) = *row_uuid;
-        error = ovsdb_file_update_row_from_json(new, converting, false, json);
+        error = ovsdb_file_update_row_from_json(new, NULL, converting,
+                                                false, json);
         if (error) {
             ovsdb_row_destroy(new);
         } else {
@@ -274,22 +285,50 @@ ovsdb_convert_table(struct ovsdb_txn *txn,
                     const struct ovsdb_table *src_table,
                     struct ovsdb_table *dst_table)
 {
+    const struct ovsdb_column **dst_columns;
+    struct ovsdb_error *error = NULL;
     const struct ovsdb_row *src_row;
+    unsigned long *src_equal;
+    struct shash_node *node;
+    size_t n_src_columns;
+
+    n_src_columns = shash_count(&src_table->schema->columns);
+    src_equal = bitmap_allocate(n_src_columns);
+    dst_columns = xzalloc(n_src_columns * sizeof *dst_columns);
+
+    SHASH_FOR_EACH (node, &src_table->schema->columns) {
+        const struct ovsdb_column *src_column = node->data;
+
+        if (src_column->index == OVSDB_COL_UUID ||
+            src_column->index == OVSDB_COL_VERSION) {
+            continue;
+        }
+
+        const struct ovsdb_column *dst_column =
+            shash_find_data(&dst_table->schema->columns, src_column->name);
+
+        if (!dst_column) {
+            continue;
+        }
+
+        dst_columns[src_column->index] = dst_column;
+
+        if (ovsdb_type_equals(&src_column->type, &dst_column->type)) {
+            bitmap_set1(src_equal, src_column->index);
+        }
+    }
+
     HMAP_FOR_EACH (src_row, hmap_node, &src_table->rows) {
         struct ovsdb_row *dst_row = ovsdb_row_create(dst_table);
         *ovsdb_row_get_uuid_rw(dst_row) = *ovsdb_row_get_uuid(src_row);
 
-        struct shash_node *node;
+        cooperative_multitasking_yield();
+
         SHASH_FOR_EACH (node, &src_table->schema->columns) {
             const struct ovsdb_column *src_column = node->data;
-            if (src_column->index == OVSDB_COL_UUID ||
-                src_column->index == OVSDB_COL_VERSION) {
-                continue;
-            }
+            const struct ovsdb_column *dst_column;
 
-            const struct ovsdb_column *dst_column
-                = shash_find_data(&dst_table->schema->columns,
-                                  src_column->name);
+            dst_column = dst_columns[src_column->index];
             if (!dst_column) {
                 continue;
             }
@@ -297,19 +336,30 @@ ovsdb_convert_table(struct ovsdb_txn *txn,
             ovsdb_datum_destroy(&dst_row->fields[dst_column->index],
                                 &dst_column->type);
 
-            struct ovsdb_error *error = ovsdb_datum_convert(
+            if (bitmap_is_set(src_equal, src_column->index)) {
+                /* This column didn't change - no need to convert. */
+                ovsdb_datum_clone(&dst_row->fields[dst_column->index],
+                                  &src_row->fields[src_column->index]);
+                continue;
+            }
+
+            error = ovsdb_datum_convert(
                 &dst_row->fields[dst_column->index], &dst_column->type,
                 &src_row->fields[src_column->index], &src_column->type);
             if (error) {
                 ovsdb_datum_init_empty(&dst_row->fields[dst_column->index]);
                 ovsdb_row_destroy(dst_row);
-                return error;
+                goto exit;
             }
         }
 
         ovsdb_txn_row_insert(txn, dst_row);
     }
-    return NULL;
+
+exit:
+    free(dst_columns);
+    bitmap_free(src_equal);
+    return error;
 }
 
 /* Copies the data in 'src', converts it into the schema specified in
@@ -489,9 +539,12 @@ ovsdb_file_txn_add_row(struct ovsdb_file_txn *ftxn,
     }
 
     if (row) {
+        ovs_assert(new || old);
         struct ovsdb_table *table = new ? new->table : old->table;
         char uuid[UUID_LEN + 1];
 
+        ovs_assert(table);
+
         if (table != ftxn->table) {
             /* Create JSON object for transaction overall. */
             if (!ftxn->json) {
