@@ -127,7 +127,8 @@ static struct ovsdb_cs_ops relay_cs_ops = {
 void
 ovsdb_relay_add_db(struct ovsdb *db, const char *remote,
                    schema_change_callback schema_change_cb,
-                   void *schema_change_aux)
+                   void *schema_change_aux,
+                   const struct jsonrpc_session_options *options)
 {
     struct relay_ctx *ctx;
 
@@ -138,6 +139,7 @@ ovsdb_relay_add_db(struct ovsdb *db, const char *remote,
     ctx = shash_find_data(&relay_dbs, db->name);
     if (ctx) {
         ovsdb_cs_set_remote(ctx->cs, remote, true);
+        ovsdb_cs_set_jsonrpc_options(ctx->cs, options);
         VLOG_DBG("%s: relay source set to '%s'", db->name, remote);
         return;
     }
@@ -152,10 +154,23 @@ ovsdb_relay_add_db(struct ovsdb *db, const char *remote,
     shash_add(&relay_dbs, db->name, ctx);
     ovsdb_cs_set_leader_only(ctx->cs, false);
     ovsdb_cs_set_remote(ctx->cs, remote, true);
+    ovsdb_cs_set_jsonrpc_options(ctx->cs, options);
 
     VLOG_DBG("added database: %s, %s", db->name, remote);
 }
 
+/* Updates the probe interval for all relay connections to the specified
+ * value. */
+void
+ovsdb_relay_set_probe_interval(int probe_interval)
+{
+    struct shash_node *node;
+    SHASH_FOR_EACH (node, &relay_dbs) {
+        struct relay_ctx *ctx = node->data;
+        ovsdb_cs_set_probe_interval(ctx->cs, probe_interval);
+    }
+}
+
 void
 ovsdb_relay_del_db(struct ovsdb *db)
 {
@@ -310,8 +325,8 @@ ovsdb_relay_parse_update(struct relay_ctx *ctx,
     if (update->monitor_reply && ctx->new_schema) {
         /* There was a schema change.  Updating a database with a new schema
          * before processing monitor reply with the new data. */
-        error = ctx->schema_change_cb(ctx->db, ctx->new_schema, false,
-                                      ctx->schema_change_aux);
+        error = ctx->schema_change_cb(ctx->db, ctx->new_schema, &UUID_ZERO,
+                                      false, ctx->schema_change_aux);
         if (error) {
             /* Should never happen, but handle this case anyway. */
             static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(1, 5);
