@@ -37,99 +37,42 @@ spellcheck = False
 quiet = False
 spell_check_dict = None
 missing_authors = []
+codespell_dictionaries = ['dictionary.txt', 'dictionary_code.txt']
+__codespell_dict_reset_once = True
 
 
 def open_spell_check_dict():
     import enchant
 
+    codespell_files = []
     try:
         import codespell_lib
         codespell_dir = os.path.dirname(codespell_lib.__file__)
-        codespell_file = os.path.join(codespell_dir, 'data', 'dictionary.txt')
-        if not os.path.exists(codespell_file):
-            codespell_file = ''
+        for fn in codespell_dictionaries:
+            fn = os.path.join(codespell_dir, 'data', fn)
+            if os.path.exists(fn):
+                codespell_files.append(fn)
     except:
-        codespell_file = ''
+        pass
 
     try:
-        extra_keywords = ['ovs', 'vswitch', 'vswitchd', 'ovs-vswitchd',
-                          'netdev', 'selinux', 'ovs-ctl', 'dpctl', 'ofctl',
-                          'openvswitch', 'dpdk', 'hugepage', 'hugepages',
-                          'pmd', 'upcall', 'vhost', 'rx', 'tx', 'vhostuser',
-                          'openflow', 'qsort', 'rxq', 'txq', 'perf', 'stats',
-                          'struct', 'int', 'char', 'bool', 'upcalls', 'nicira',
-                          'bitmask', 'ipv4', 'ipv6', 'tcp', 'tcp4', 'tcpv4',
-                          'udp', 'udp4', 'udpv4', 'icmp', 'icmp4', 'icmpv6',
-                          'vlan', 'vxlan', 'cksum', 'csum', 'checksum',
-                          'ofproto', 'numa', 'mempool', 'mempools', 'mbuf',
-                          'mbufs', 'hmap', 'cmap', 'smap', 'dhcpv4', 'dhcp',
-                          'dhcpv6', 'opts', 'metadata', 'geneve', 'mutex',
-                          'netdev', 'netdevs', 'subtable', 'virtio', 'qos',
-                          'policer', 'datapath', 'tunctl', 'attr', 'ethernet',
-                          'ether', 'defrag', 'defragment', 'loopback', 'sflow',
-                          'acl', 'initializer', 'recirc', 'xlated', 'unclosed',
-                          'netlink', 'msec', 'usec', 'nsec', 'ms', 'us', 'ns',
-                          'kilobits', 'kbps', 'kilobytes', 'megabytes', 'mbps',
-                          'gigabytes', 'gbps', 'megabits', 'gigabits', 'pkts',
-                          'tuple', 'miniflow', 'megaflow', 'conntrack',
-                          'vlans', 'vxlans', 'arg', 'tpid', 'xbundle',
-                          'xbundles', 'mbundle', 'mbundles', 'netflow',
-                          'localnet', 'odp', 'pre', 'dst', 'dest', 'src',
-                          'ethertype', 'cvlan', 'ips', 'msg', 'msgs',
-                          'liveness', 'userspace', 'eventmask', 'datapaths',
-                          'slowpath', 'fastpath', 'multicast', 'unicast',
-                          'revalidation', 'namespace', 'qdisc', 'uuid',
-                          'ofport', 'subnet', 'revalidation', 'revalidator',
-                          'revalidate', 'l2', 'l3', 'l4', 'openssl', 'mtu',
-                          'ifindex', 'enum', 'enums', 'http', 'https', 'num',
-                          'vconn', 'vconns', 'conn', 'nat', 'memset', 'memcmp',
-                          'strcmp', 'strcasecmp', 'tc', 'ufid', 'api',
-                          'ofpbuf', 'ofpbufs', 'hashmaps', 'hashmap', 'deref',
-                          'dereference', 'hw', 'prio', 'sendmmsg', 'sendmsg',
-                          'malloc', 'free', 'alloc', 'pid', 'ppid', 'pgid',
-                          'uid', 'gid', 'sid', 'utime', 'stime', 'cutime',
-                          'cstime', 'vsize', 'rss', 'rsslim', 'whcan', 'gtime',
-                          'eip', 'rip', 'cgtime', 'dbg', 'gw', 'sbrec', 'bfd',
-                          'sizeof', 'pmds', 'nic', 'nics', 'hwol', 'encap',
-                          'decap', 'tlv', 'tlvs', 'decapsulation', 'fd',
-                          'cacheline', 'xlate', 'skiplist', 'idl',
-                          'comparator', 'natting', 'alg', 'pasv', 'epasv',
-                          'wildcard', 'nated', 'amd64', 'x86_64',
-                          'recirculation', 'linux', 'afxdp', 'promisc', 'goto',
-                          'misconfigured', 'misconfiguration', 'checkpatch',
-                          'debian', 'travis', 'cirrus', 'appveyor', 'faq',
-                          'erspan', 'const', 'hotplug', 'addresssanitizer',
-                          'ovsdb', 'dpif', 'veth', 'rhel', 'jsonrpc', 'json',
-                          'syscall', 'lacp', 'ipf', 'skb', 'valgrind',
-                          'appctl', 'arp', 'asan', 'backport', 'backtrace',
-                          'chmod', 'ci', 'cpu', 'cpus', 'dnat', 'dns', 'dpcls',
-                          'eol', 'ethtool', 'fdb', 'freebsd', 'gcc', 'github',
-                          'glibc', 'gre', 'inlined', 'ip', 'ipfix', 'ipsec',
-                          'ixgbe', 'libbpf', 'libcrypto', 'libgcc',
-                          'libopenvswitch', 'libreswan', 'libssl', 'libxdp',
-                          'lldp', 'llvm', 'lockless', 'mcast', 'megaflows',
-                          'mfex', 'ncat', 'networkmanager', 'pcap', 'pedit',
-                          'pidfile', 'pps', 'rculist', 'rebalance', 'rebased'
-                          'recirculations', 'revalidators', 'rst', 'sed',
-                          'shrinked', 'snat', 'stderr', 'stdout', 'testpmd',
-                          'tftp', 'timeval', 'trie', 'tso', 'ubsan', 'ukey',
-                          'umask', 'unassociated', 'unixctl', 'uuid'
-                          'virtqueue', 'vms', 'vnet', 'vport', 'vports',
-                          'vtep', 'wc', 'wget', 'xenserver']
-
         global spell_check_dict
 
         spell_check_dict = enchant.Dict("en_US")
 
-        if codespell_file:
-            with open(codespell_file) as f:
+        for fn in codespell_files:
+            with open(fn) as f:
                 for line in f.readlines():
                     words = line.strip().split('>')[1].strip(', ').split(',')
                     for word in words:
                         spell_check_dict.add_to_session(word.strip())
 
-        for kw in extra_keywords:
-            spell_check_dict.add_to_session(kw)
+        check_dict_file = os.path.join(get_top_directory(), "utilities",
+                                       "checkpatch_dict.txt")
+        if os.path.exists(check_dict_file):
+            with open(check_dict_file, "r") as project_words:
+                for check_word in project_words:
+                    spell_check_dict.add_to_session(check_word.strip())
 
         return True
     except:
@@ -1130,6 +1073,7 @@ Check options:
 -a|--check-authors-file        Check AUTHORS file for existence of the authors.
                                Should be used by commiters only!
 -b|--skip-block-whitespace     Skips the if/while/for whitespace tests
+-D|--dictionaries DICTIONARY   Specify codespell dictionaries.
 -l|--skip-leading-whitespace   Skips the leading whitespace test
 -q|--quiet                     Only print error and warning information
 -s|--skip-signoff-lines        Tolerate missing Signed-off-by line
@@ -1217,10 +1161,11 @@ if __name__ == '__main__':
                                           sys.argv[1:])
         n_patches = int(numeric_options[-1][1:]) if numeric_options else 0
 
-        optlist, args = getopt.getopt(args, 'abhlstfSq',
+        optlist, args = getopt.getopt(args, 'abD:hlstfSq',
                                       ["check-file",
                                        "help",
                                        "check-authors-file",
+                                       "dictionaries=",
                                        "skip-block-whitespace",
                                        "skip-leading-whitespace",
                                        "skip-signoff-lines",
@@ -1239,6 +1184,11 @@ if __name__ == '__main__':
             sys.exit(0)
         elif o in ("-a", "--check-authors-file"):
             check_authors_file = True
+        elif o in ("-D", "--dictionaries"):
+            if __codespell_dict_reset_once:
+                __codespell_dict_reset_once = False
+                codespell_dictionaries = []
+            codespell_dictionaries.append(a)
         elif o in ("-b", "--skip-block-whitespace"):
             skip_block_whitespace_check = True
         elif o in ("-l", "--skip-leading-whitespace"):
@@ -1254,11 +1204,7 @@ if __name__ == '__main__':
         elif o in ("-f", "--check-file"):
             checking_file = True
         elif o in ("-S", "--spellcheck"):
-            if not open_spell_check_dict():
-                print("WARNING: The enchant library isn't available.")
-                print("         Please install python enchant.")
-            else:
-                spellcheck = True
+            spellcheck = True
         elif o in ("-q", "--quiet"):
             quiet = True
         else:
@@ -1268,6 +1214,11 @@ if __name__ == '__main__':
     if sys.stdout.isatty():
         colors = True
 
+    if spellcheck and not open_spell_check_dict():
+        print("WARNING: The enchant library isn't available.")
+        print("         Please install python enchant.")
+        spellcheck = False
+
     if n_patches:
         status = 0
 
